#! /usr/bin/perl
# encode / decode QTPI phone books
# gratuitously perl 4 compatible for QDOS, alas
#
# qtpiph.pl File.name
#
# if File.name exists, it is decoded to stdout
# if File_name does not exist, it is created from stdin
#
# hence (assuming/ql/qlsoft/qbook.dat exists)  :
# qtpiph.pl  /ql/qlsoft/qbook.dat |  qtpiph.pl /tmp/xx.dat
# then 0 == diff   /ql/qlsoft/qbook.dat /tmp/xx.dat
#
# (c) 1999 Jonathan Hudson <jrhudson@bigfoot.com>
# Distributed under the GNU Public License

sub fixup
{
  $_[0] =~ /\0/;
  $_[0] = $`;
  $_[0] =~ s/(\s*)(\S+)/$2/;

} 
sub GetArry
{
  ($wanted, @arry) = @_;
  $idx = 0;
  foreach (@arry)
  {
    last if $_ eq $wanted;
    $idx++;
  }
  $idx = 0 if $idx > $#arry;
  $idx;
}

@emus = ('VT52','VT100','VT102','ANSI1','ANSI2','ANSI3','ANSI4');
@eols = ('EOLLF','EOLCR','EOLCRLF'); 
@bauds =
('75','300','600','1200','2400','4800','9600','19200','38400','57600','None');
@delc = ('Del','BS'); 
@parity = ('None','Even','Odd','Space','Mark','CIS');
@flow = ('Handshake','Ignore');
@key7 = ('No','Yes');

die "No file" unless $#ARGV == 0;


if(-e $ARGV[0])
{
  open(F,$ARGV[0]) || die "Can't open your file ";
  sysread(F,$vers,6) || die;
  die unless $vers =~ /^QeM v4/;
  
  while(sysread(F,$buf,256) == 256)
  {
    ($name,$phone,$logon,$pass,$cmnt,$mdm,@flags,$proto) = 
      unpack ("a32a40a40a40a40a32c16a16", $buf);
    
    &fixup($name);
    &fixup($phone);
    &fixup($logon);
    &fixup($pass);
    &fixup($cmnt);
    &fixup($mdm);
    &fixup($proto);
    
    print "BBS: ", $name,"\n";
    print "\tPhone: ", $phone,"\n";
    print "\tLogon: ", $logon,"\n";
    print "\tPass: ", $pass,"\n";
    print "\tComment: ", $cmnt,"\n";
    print "\tModem: ", $mdm,"\n";
    print "\tProtocol: ", $proto,"\n";
    print "\tEmulation: ",$emus[$flags[0]], "\n";
    print "\tBaud Rate: ",$bauds[$flags[10]], "\n";
    print "\tParity: ",$parity[$flags[11]], "\n";
    print "\tFlow Control: ",$flow[$flags[12]], "\n";
    print "\tRX EOL: ",$eols[$flags[7]], "\n";
    print "\tTX EOL: ",$eols[$flags[8]], "\n";
    print "\tDel Char: ",$delc[$flags[9]], "\n";
    print "\t7 Bit keys: ",$key7[$flags[13]], "\n";
  }
}
else
{
  undef $name;
  open(F,">".$ARGV[0]) || die "Can't open your file ";
  print F "QeM v4";
  while(<STDIN>)
  {
    chop;
    SW1:
    {
      /^BBS:\s+(.*)/ && do
      {
	$tmp = $1;
	if($name)
	{
	  $buf = pack ("a32a40a40a40a40a32c16a16", 
		       ($name,$phone,$logon,$pass,$cmnt,$mdm,
			@flags, $proto));
	  print F $buf;
	  undef $phone;
	  undef $logon;
	  undef $pass;
	  undef $cmnt;
	  undef $mdm;
	  undef $proto;
	  undef @flags;
	}
	$name = $tmp;
	last SW1;
      };

      /^\s+Phone:\s+(.*)/ && do
      {
	$phone = $1;
	last SW1;
      };

      /^\s+Logon:\s+(.*)/ && do
      {
	$logon = $1;
	last SW1;
      };

      /^\s+Pass:\s+(.*)/ && do
      {
	$pass = $1;
	last SW1;
      };

      /^\s+Comment:\s+(.*)/ && do
      {
	$cmnt = $1;
	last SW1;
      };

      /^\s+Modem:\s+(.*)/ && do
      {
	$mdm = $1;
	last SW1;
      };

      /^\s+Protocol:\s+(.*)/ && do
      {
	$proto = $1;
	last SW1;
      };

      /^\s+Emulation:\s+(.*)/ && do
      {
	$flags[0] = &GetArry($1, @emus);
	last SW1;
      };

      /^\s+Baud Rate:\s+(.*)/ && do
      {
	$flags[10] = &GetArry($1, @bauds);
	last SW1;
      };

      /^\s+Parity:\s+(.*)/ && do
      {
	$flags[11] = &GetArry($1, @parity);
	last SW1;
      };

      /^\s+Flow Control:\s+(.*)/ && do
      {
	$flags[12] = &GetArry($1, @flow);
	last SW1;
      };

      /^\s+RX EOL:\s+(.*)/ && do
      {
	$flags[7] = &GetArry($1, @eols);
	last SW1;
      };

      /^\s+TX EOL:\s+(.*)/ && do
      {
	$flags[8] = &GetArry($1, @eols);
	last SW1;
      };

      /^\s+Del Char:\s+(.*)/ && do
      {
	$flags[9] = &GetArry($1, @delc);
	last SW1;
      };

      /^\s+7 Bit keys:\s+(.*)/ && do
      {
	$flags[13] = &GetArry($1, @key7);
	last SW1;
      };
      die $_." You've screwed it up\n";
    }
  }
  if($name)
  {
    $buf = pack ("a32a40a40a40a40a32c16a16", 
		 ($name,$phone,$logon,$pass,$cmnt,$mdm,
		  @flags, $proto));
    print F $buf;
  }
  close F;
}
